@@ -27,7 +27,7 @@
 #ifndef WIN32
     #include  <sys/time.h>
 #else
-    #include  <QTime>
+    #include  <ctime>
 #endif
 
 #include  <QHttp>
@@ -78,6 +78,7 @@ AlbumInfoDownloaderDlgImpl::~AlbumInfoDownloaderDlgImpl()
 
 bool AlbumInfoDownloaderDlgImpl::getInfo(const std::string& strArtist, const std::string& strAlbum, int nTrackCount, AlbumInfo*& pAlbumInfo, ImageInfo*& pImageInfo)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::getInfo");
     m_nExpectedTracks = nTrackCount;
 
     pAlbumInfo = 0; pImageInfo = 0;
@@ -96,6 +97,8 @@ bool AlbumInfoDownloaderDlgImpl::getInfo(const std::string& strArtist, const std
         {
             pAlbumInfo = new AlbumInfo();
             album(m_nCrtAlbum).copyTo(*pAlbumInfo);
+            //pAlbumInfo->m_strReleased = convStr(m_pRealeasedE->text()); //ttt2 maybe allow users to overwrite fields in edit boxes, esp. genre; however, most of the cases it's almost as easy to make any changes in the tag editor (well, there's an F2 and then "copy form first"); there are several issues in implementing this: 1) going to prev/next album; 2) artist name gets copied to each track; 3) consistency: if the edit boxes are editable why not the table? so, better without
+
             if (m_pVolumeCbB->isEnabled() && m_pVolumeCbB->currentIndex() != m_pVolumeCbB->count() - 1)
             {
                 vector<TrackInfo> vTracks;
@@ -125,16 +128,34 @@ bool AlbumInfoDownloaderDlgImpl::getInfo(const std::string& strArtist, const std
 // clears pending HTTP requests, m_eNavigDir and m_eWaiting; restores the cursor if needed;
 /*virtual*/ void AlbumInfoDownloaderDlgImpl::resetNavigation()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::resetNavigation");
     m_pQHttp->clearPendingRequests();
     setWaiting(NOTHING);
     m_eNavigDir = NONE;
 }
 
 
+
+string AlbumInfoDownloaderDlgImpl::replaceSymbols(string s) // replaces everything besides letters and digits with getReplacementChar()
+{
+LAST_STEP("AlbumInfoDownloaderDlgImpl::replaceSymbols");
+    char c (getReplacementChar());
+    for (int i = 0; i < cSize(s); ++i)
+    {
+        if ((unsigned char)(s[i]) < 128 && '\'' != s[i] && !isalnum(s[i]))
+        {
+            s[i] = c;
+        }
+    }
+
+    return s;
+}
+
 /*static*/ const char* AlbumInfoDownloaderDlgImpl::NOT_FOUND_AT_AMAZON = "not found at amazon.com";
 
 void AlbumInfoDownloaderDlgImpl::search()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::search");
     m_pResArtistE->setText("");
     m_pResAlbumE->setText("");
     m_pDownloadsM->setText("");
@@ -165,6 +186,7 @@ void AlbumInfoDownloaderDlgImpl::search()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pPrevB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pPrevB_clicked");
     if (0 == getAlbumCount() || NOTHING != m_eWaiting) { return; }
 
     m_bNavigateByAlbum = false;
@@ -175,6 +197,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pPrevB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pNextB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pNextB_clicked");
     if (0 == getAlbumCount() || NOTHING != m_eWaiting) { return; }
 
     m_bNavigateByAlbum = false;
@@ -185,6 +208,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pNextB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pPrevAlbumB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pPrevAlbumB_clicked");
     if (0 == getAlbumCount() || NOTHING != m_eWaiting) { return; }
 
     m_bNavigateByAlbum = true;
@@ -195,6 +219,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pPrevAlbumB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pNextAlbumB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pNextAlbumB_clicked");
     if (0 == getAlbumCount() || NOTHING != m_eWaiting) { return; }
 
     m_bNavigateByAlbum = true;
@@ -205,6 +230,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pNextAlbumB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pSaveAllB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pSaveAllB_clicked");
     if (NOTHING != m_eWaiting)
     {
         QMessageBox::critical(this, "Error", "You cannot save the results now, because a request is still pending");
@@ -265,6 +291,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pSaveAllB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pSaveImageB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pSaveImageB_clicked");
     if (NOTHING != m_eWaiting)
     {
         QMessageBox::critical(this, "Error", "You cannot save the results now, because a request is still pending");
@@ -276,7 +303,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pSaveImageB_clicked()
         QMessageBox::critical(this, "Error", "You cannot save any image now, because there is no image loaded");
         return;
     }
-    // ttt1 perhaps shouldn't save an "error" image
+    // ttt2 perhaps shouldn't save an "error" image
 
     m_bSaveImageOnly = true;
     accept();
@@ -284,6 +311,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pSaveImageB_clicked()
 
 void AlbumInfoDownloaderDlgImpl::on_m_pCancelB_clicked()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::on_m_pCancelB_clicked");
     reject();
 }
 
@@ -304,6 +332,7 @@ void AlbumInfoDownloaderDlgImpl::on_m_pCancelB_clicked()
 //
 void AlbumInfoDownloaderDlgImpl::next()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::next");
     //if (NONE != m_eNavigDir) { return; }
     CB_ASSERT (NEXT == m_eNavigDir);
 
@@ -346,7 +375,7 @@ void AlbumInfoDownloaderDlgImpl::next()
 
         if ((m_pImageFltCkB->isChecked() && album(nNextAlbum).m_vpImages.empty()) ||
             (m_pCdFltCkB->isChecked() && string::npos == album(nNextAlbum).m_strFormat.find("CD")) ||
-            (m_pTrackCntFltCkB->isChecked() && m_nExpectedTracks != cSize(album(nNextAlbum).m_vTracks))) // ttt1 MusicBrainz could perform better here, because it knows how many tracks are in an album without actually loading it; ttt1 redo the whole next() / prev() thing in a more logical way; perhaps separate next() from nextAlbum();
+            (m_pTrackCntFltCkB->isChecked() && m_nExpectedTracks != cSize(album(nNextAlbum).m_vTracks))) // ttt2 MusicBrainz could perform better here, because it knows how many tracks are in an album without actually loading it; ttt2 redo the whole next() / prev() thing in a more logical way; perhaps separate next() from nextAlbum();
         {
             continue;
         }
@@ -363,13 +392,14 @@ void AlbumInfoDownloaderDlgImpl::next()
         return;
     }
 }
-//ttt1 perhaps filter by durations (but see first how it works without it)
+//ttt2 perhaps filter by durations (but see first how it works without it)
 
 
 
 // like next(), but here there's no need to load result pages; another difference is that when m_bNavigateByAlbum is set and we are at the first album and some other picture than the first, it goes to the first picture; in the similar case, next doesn't do anything (going to the last picture doesn't feel right)
 void AlbumInfoDownloaderDlgImpl::previous()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::previous");
     CB_ASSERT (PREV == m_eNavigDir);
     //if (NONE != m_eNavigDir) { return; }
 
@@ -408,7 +438,7 @@ void AlbumInfoDownloaderDlgImpl::previous()
         // if it got here, we have a loaded album (pictures might be missing, though)
 
         if ((m_pImageFltCkB->isChecked() && album(nPrevAlbum).m_vpImages.empty()) ||
-            (m_pCdFltCkB->isChecked() && string::npos == album(nPrevAlbum).m_strFormat.find("CD")) || // Discogs has one format, but MusicBrainz may have several, so find() is used instead of "==" 
+            (m_pCdFltCkB->isChecked() && string::npos == album(nPrevAlbum).m_strFormat.find("CD")) || // Discogs has one format, but MusicBrainz may have several, so find() is used instead of "=="
             (m_pTrackCntFltCkB->isChecked() && m_nExpectedTracks != cSize(album(nPrevAlbum).m_vTracks)))
         {
             continue;
@@ -430,6 +460,7 @@ void AlbumInfoDownloaderDlgImpl::previous()
 
 void AlbumInfoDownloaderDlgImpl::setImageType(const string& strName)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::setImageType");
     m_eLoadingImageCompr = ImageInfo::INVALID;
     string::size_type m (strName.rfind('.'));
     if (string::npos != m)
@@ -452,10 +483,11 @@ void AlbumInfoDownloaderDlgImpl::setImageType(const string& strName)
     }
 }
 
-//ttt1 2009.05.18 - once the "format" edit in MusicBrainz dwnld was very narrow; it was OK after the program was restarted, and the issue didn't repeat again
+
 
 void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::onRequestFinished");
 //cout << "received ID = " << nId << endl;
 //if (1 == nId) { return; } // some automatically generated request, which should be ignored
     if (bError)
@@ -479,7 +511,7 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
 
     CB_ASSERT (NOTHING != m_eWaiting);
 
-    { QString qstrMsg (QString("received %1 bytes").arg(nAv)); addNote(qstrMsg.toLatin1().data()); }
+    { QString qstrMsg (QString("received %1 bytes").arg(nAv)); addNote(qstrMsg.toLatin1().constData()); }
 
     QString qstrXml;
 
@@ -496,8 +528,10 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
     if (IMAGE == m_eWaiting)
     {
         QString qstrInfo;
-        QPixmap img;
+        QImage img;
         QByteArray comprImg (b);
+        ImageInfo::Compr eOrigCompr (m_eLoadingImageCompr);
+
         if (img.loadFromData(b)) //ttt2 not sure what happens for huge images;
         {
             qstrInfo = QString("Original: %1kB, %2x%3").arg(nAv/1024).arg(img.width()).arg(img.height());
@@ -507,7 +541,7 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
 
             if (nAv > ImageInfo::MAX_IMAGE_SIZE || m_eLoadingImageCompr == ImageInfo::INVALID)
             {
-                QPixmap scaledImg;
+                QImage scaledImg;
                 ImageInfo::compress(img, scaledImg, comprImg);
                 nWidth = scaledImg.width(); nHeight = scaledImg.height();
                 m_eLoadingImageCompr = ImageInfo::JPG;
@@ -525,11 +559,11 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
         {
             QMessageBox::critical(this, "Error", "Failed to load the image");
             const int SIZE (150);
-            QPixmap errImg (SIZE, SIZE);
+            QImage errImg (SIZE, SIZE, QImage::Format_ARGB32);
             QPainter pntr (&errImg);
             pntr.fillRect(0, 0, SIZE, SIZE, QColor(255, 128, 128));
             pntr.drawRect(0, 0, SIZE - 1, SIZE - 1);
-            pntr.drawText(5, SIZE/2 + 10, "Error");
+            pntr.drawText(QRectF(0, 0, SIZE, SIZE), Qt::AlignCenter, "Error");
             qstrInfo = "Error loading image\n";
             comprImg.clear();
             QBuffer bfr (&comprImg);
@@ -538,7 +572,7 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
             onImageLoaded(comprImg, SIZE, SIZE, qstrInfo);
         }
 
-        if (m_bSaveResults) { saveDownloadedData(b.data(), b.size(), ImageInfo::JPG == m_eLoadingImageCompr ? "jpg" : "png"); }
+        if (m_bSaveResults) { saveDownloadedData(b.constData(), b.size(), (ImageInfo::JPG == eOrigCompr ? "jpg" : (ImageInfo::PNG == eOrigCompr ? "png" : "unkn"))); }
 
         return;
     }
@@ -549,7 +583,7 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
         strm.zalloc = Z_NULL;
         strm.zfree  = Z_NULL;
         strm.opaque = 0;
-        strm.next_in = reinterpret_cast<unsigned char*>(b.data());
+        strm.next_in = const_cast<unsigned char*>(reinterpret_cast<const unsigned char*>(b.constData()));
         strm.avail_in = nAv;
 
         vector<char> v (nAv);
@@ -596,13 +630,14 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
     if (qstrXml.isEmpty())
     {
         addNote("empty string received");
-        return;
     }
-
-    if (m_bSaveResults)
+    else
     {
-        QByteArray b1 (qstrXml.toUtf8());
-        saveDownloadedData(b1.data(), b1.size(), "xml");
+        if (m_bSaveResults)
+        {
+            QByteArray b1 (qstrXml.toUtf8());
+            saveDownloadedData(b1.constData(), b1.size(), "xml");
+        }
     }
 
     switch (m_eWaiting)
@@ -623,6 +658,7 @@ void AlbumInfoDownloaderDlgImpl::onRequestFinished(int /*nId*/, bool bError)
 
 string AlbumInfoDownloaderDlgImpl::getTempName() // time-based, with no extension; doesn't check for existing names, but uses a counter, so files shouldn't get removed (except during daylight saving time changes)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::getTempName");
     time_t t (time(0));
     if (t == m_nLastTime)
     {
@@ -656,6 +692,7 @@ string AlbumInfoDownloaderDlgImpl::getTempName() // time-based, with no extensio
 
 void AlbumInfoDownloaderDlgImpl::retryNavigation()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::retryNavigation");
     if (NEXT == m_eNavigDir)
     {
         next();
@@ -673,12 +710,13 @@ void AlbumInfoDownloaderDlgImpl::retryNavigation()
     {
         resetNavigation();
     }
-    // ttt1 perhaps add assert that either there are no pending requests and NOTHING==m_eWaiting or there is 1 pending request and NOTHING!=m_eWaiting (keep in mind that when the connection is opened there is a system-generated request);
+    // ttt2 perhaps add assert that either there are no pending requests and NOTHING==m_eWaiting or there is 1 pending request and NOTHING!=m_eWaiting (keep in mind that when the connection is opened there is a system-generated request);
 }
 
 
 void AlbumInfoDownloaderDlgImpl::onSearchLoaded(const QString& qstrXml)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::onSearchLoaded");
     addNote("search results received");
     QByteArray b (qstrXml.toLatin1());
     QBuffer bfr (&b);
@@ -702,7 +740,7 @@ void AlbumInfoDownloaderDlgImpl::onSearchLoaded(const QString& qstrXml)
         return;
     }
 
-    if (0 == getAlbumCount())
+    if (0 == getAlbumCount() && m_nLastLoadedPage == m_nTotalPages - 1)
     {
         QMessageBox::critical(this, "Error", "No results found");
     }
@@ -713,6 +751,7 @@ void AlbumInfoDownloaderDlgImpl::onSearchLoaded(const QString& qstrXml)
 
 void AlbumInfoDownloaderDlgImpl::onAlbumLoaded(const QString& qstrXml)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::onAlbumLoaded");
     addNote("album info received");
     QByteArray b (qstrXml.toLatin1());
     QBuffer bfr (&b);
@@ -744,6 +783,7 @@ void AlbumInfoDownloaderDlgImpl::onAlbumLoaded(const QString& qstrXml)
 
 void AlbumInfoDownloaderDlgImpl::onImageLoaded(const QByteArray& comprImg, int nWidth, int nHeight, const QString& qstrInfo)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::onImageLoaded");
     addNote("image received");
     CB_ASSERT (0 == album(m_nLoadingAlbum).m_vpImages[m_nLoadingImage]);
     CB_ASSERT (ImageInfo::INVALID != m_eLoadingImageCompr);
@@ -759,6 +799,7 @@ void AlbumInfoDownloaderDlgImpl::onImageLoaded(const QByteArray& comprImg, int n
 
 void AlbumInfoDownloaderDlgImpl::addNote(const char* szNote)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::addNote");
     QString q (m_pDownloadsM->toPlainText());
     if (!q.isEmpty()) { q += "\n"; }
     {
@@ -781,6 +822,7 @@ void AlbumInfoDownloaderDlgImpl::addNote(const char* szNote)
 
 void AlbumInfoDownloaderDlgImpl::setWaiting(Waiting eWaiting)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::setWaiting");
     if (NOTHING == m_eWaiting && NOTHING != eWaiting)
     {
         QApplication::setOverrideCursor(QCursor(Qt::BusyCursor));
@@ -797,6 +839,7 @@ void AlbumInfoDownloaderDlgImpl::setWaiting(Waiting eWaiting)
 
 void AlbumInfoDownloaderDlgImpl::reloadGui()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::reloadGui");
     resetNavigation();
 
     if (0 == getAlbumCount()) { return; }
@@ -854,7 +897,7 @@ void AlbumInfoDownloaderDlgImpl::reloadGui()
     }
     else
     {
-        m_pImageL->setPixmap(albumInfo.m_vpImages[m_nCrtImage]->getPixmap(m_pImageL->width(), m_pImageL->height()));
+        m_pImageL->setPixmap(QPixmap::fromImage(albumInfo.m_vpImages[m_nCrtImage]->getImage(m_pImageL->width(), m_pImageL->height())));
         m_pImgSizeL->setText(convStr(albumInfo.m_vstrImageInfo[m_nCrtImage]));
     }
 }
@@ -862,6 +905,7 @@ void AlbumInfoDownloaderDlgImpl::reloadGui()
 
 /*override*/ void AlbumInfoDownloaderDlgImpl::updateTrackList()
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::updateTrackList");
     m_pModel->emitLayoutChanged();
 
     SimpleQTableViewWidthInterface intf (*m_pTrackListG);
@@ -873,21 +917,45 @@ void AlbumInfoDownloaderDlgImpl::reloadGui()
 /*static*/ string AlbumInfoDownloaderDlgImpl::removeParentheses(const string& s)
 {
     string r;
-    int k (0);
+    int k1 (0), k2 (0), k3 (0), k4 (0);
     for (int i = 0, n = cSize(s); i < n; ++i)
     {
         char c (s[i]);
         if ('(' == c)
         {
-            ++k;
+            ++k1;
         }
-        if (0 == k)
+        if ('[' == c)
+        {
+            ++k2;
+        }
+        if ('{' == c)
+        {
+            ++k3;
+        }
+        if ('<' == c)
+        {
+            ++k3;
+        }
+        if (0 == k1 && 0 == k2 && 0 == k3 && 0 == k4)
         {
             r += c;
         }
         if (')' == c)
         {
-            --k;
+            --k1;
+        }
+        if (']' == c)
+        {
+            --k2;
+        }
+        if ('}' == c)
+        {
+            --k3;
+        }
+        if ('>' == c)
+        {
+            --k4;
         }
     }
     trim(r);
@@ -897,6 +965,7 @@ void AlbumInfoDownloaderDlgImpl::reloadGui()
 
 void AlbumInfoDownloaderDlgImpl::saveDownloadedData(const char* p, int nSize, const char* szExt)
 {
+LAST_STEP("AlbumInfoDownloaderDlgImpl::saveDownloadedData");
     string s (getTempName() + "." + szExt);
     ofstream_utf8 out (s.c_str(), ios::binary);
     out.write(p, nSize);
@@ -1000,7 +1069,7 @@ WebDwnldModel::WebDwnldModel(AlbumInfoDownloaderDlgImpl& dwnld, QTableView& grid
 
 /*override*/ QVariant WebDwnldModel::data(const QModelIndex& index, int nRole) const
 {
-LAST_STEP("WebDwnldModel::data()");
+//LAST_STEP("WebDwnldModel::data()");
     if (!index.isValid()) { return QVariant(); }
     if (nRole != Qt::DisplayRole && nRole != Qt::ToolTipRole) { return QVariant(); }
     int i (index.row()), j (index.column());
@@ -1065,4 +1134,3 @@ LAST_STEP("WebDwnldModel::data()");
 }
 
 
-
