@@ -20,6 +20,8 @@
  ***************************************************************************/
 
 
+#include  <map>
+
 #include  <QHttp>
 
 #include  "DiscogsDownloader.h"
@@ -38,7 +40,7 @@ using namespace Discogs;
 
 ostream& operator<<(ostream& out, const DiscogsAlbumInfo& inf)
 {
-    out << "id: \"" << inf.m_strId << "\", artist: \"" << inf.m_strArtist << "\", title: \"" << inf.m_strTitle << "\", composer: \"" << inf.m_strComposer << "\", format: \"" << inf.m_strFormat << "\", genre: \"" << inf.m_strGenre << "\", released: \"" << inf.m_strReleased << "\"\n\nnotes: " << inf.m_strNotes << "\n\nimages:\n";
+    out << "id: \"" << inf.m_strId << "\", artist: \"" << inf.m_strArtist << "\", title: \"" << inf.m_strTitle << "\", composer: \"" << inf.m_strComposer << "\", format: \"" << inf.m_strFormat << "\", genre: \"" << inf.m_strGenre << "\", style: \"" << inf.m_strStyle << "\", released: \"" << inf.m_strReleased << "\"\n\nnotes: " << inf.m_strNotes << "\n\nimages:\n";
 
     for (int i = 0, n = cSize(inf.m_vstrImageNames); i < n; ++i)
     {
@@ -91,7 +93,7 @@ private:
         int nStart (attrs.value("start").toInt());
         int nEnd (attrs.value("end").toInt());
         int nCount (attrs.value("numResults").toInt());
-        CB_ASSERT (0 == nEnd % 20 || nEnd == nCount); //ttt1 see if this needs to be more flexible
+        CB_ASSERT (0 == nEnd % 20 || nEnd == nCount); //ttt2 see if this needs to be more flexible
         int nPageCnt ((nCount + 19) / 20);
         int nCrtPage ((nStart - 1)/20);
         if (0 == nPageCnt) { nCrtPage = -1; } // !!! needed for consistency
@@ -105,7 +107,7 @@ private:
         m_bIsRelease = "release" == attrs.value("type");
         if (m_bIsRelease)
         {
-            m_dlg.m_vAlbums.push_back(DiscogsAlbumInfo());
+            m_dlg.m_vAlbums.push_back(DiscogsAlbumInfo(&m_dlg.m_eStyleOption));
         }
     }
 
@@ -144,8 +146,8 @@ private:
                 format
             genres
                 genre
-            //styles
-                //style
+            styles
+                style
             released
             notes
             tracklist
@@ -181,8 +183,8 @@ struct AlbumXmlHandler : public SimpleSaxHandler<AlbumXmlHandler>
                     Node& format (makeNode(formats, "format")); format.onStart = &AlbumXmlHandler::onFormatStart;
                 Node& genres (makeNode(rel, "genres"));
                     Node& genre (makeNode(genres, "genre")); genre.onChar = &AlbumXmlHandler::onGenreChar;
-                //Node& QQQQQQQ (makeNode(QQQQQQQ, "styles //ttt1 see if there's a use for styles
-                    //Node& QQQQQQQ (makeNode(QQQQQQQ, "style
+                Node& styles (makeNode(rel, "styles"));
+                    Node& style (makeNode(styles, "style")); style.onChar = &AlbumXmlHandler::onStyleChar;
                 Node& released (makeNode(rel, "released")); released.onChar = &AlbumXmlHandler::onReleasedChar;
                 Node& notes (makeNode(rel, "notes")); notes.onChar = &AlbumXmlHandler::onNotesChar;
                 Node& tracklist (makeNode(rel, "tracklist"));
@@ -238,7 +240,7 @@ private:
         //qDebug(" orig role %s", s.c_str());
         s = transfRole(s);
         //qDebug(" transf role %s", s.c_str());
-        return "composedby" == s || "writtenby" == s || "musicby" == s || "lyricsby" == s; // ttt1 review whether all these should be added to "composer"; see if some others should be put here
+        return "composedby" == s || "writtenby" == s || "musicby" == s || "lyricsby" == s; // ttt2 review whether all these should be added to "composer"; see if some others should be put here
     }
 
 
@@ -344,7 +346,7 @@ private:
 
     void onTrkExtraArtistEnd()
     {
-        if (isComposerRole(m_strExtraArtistRole)) // ttt1 see if "Music By" should be compared too
+        if (isComposerRole(m_strExtraArtistRole)) // ttt2 see if "Music By" should be compared too
         {
             addIfMissing(m_albumInfo.m_vTracks.back().m_strComposer, m_strExtraArtistName);
 //inspect(m_strExtraArtistName);
@@ -359,7 +361,7 @@ private:
     void onExtraArtistNameChar(const string& s)
     {
         m_strExtraArtistName = s;
-        //addIfMissing(m_strExtraArtistName, s); // ttt1 review if this makes sense, given that for artists there is the option to join, but not sure about extraartists:
+        //addIfMissing(m_strExtraArtistName, s); // ttt2 review if this makes sense, given that for artists there is the option to join, but not sure about extraartists:
         /*
         <track>
             <position>5</position>
@@ -408,6 +410,11 @@ private:
         addIfMissing(m_albumInfo.m_strGenre, s);
     }
 
+    void onStyleChar(const string& s)
+    {
+        addIfMissing(m_albumInfo.m_strStyle, s);
+    }
+
     void onReleasedChar(const string& s)
     {
         m_albumInfo.m_strReleased = s;
@@ -448,16 +455,35 @@ private:
     //dest.m_strArtist = m_strArtist;
     //dest.m_strComposer = m_strComposer;
     //dest.m_strFormat = m_strFormat; // CD, tape, ...
-    dest.m_strGenre = m_strGenre;
+    dest.m_strGenre = getGenre();
     dest.m_strReleased = m_strReleased;
     dest.m_strNotes = m_strNotes;
     dest.m_vTracks = m_vTracks;
+    //dest.m_eVarArtists  // !!! missing
 
     dest.m_strSourceName = DiscogsDownloader::SOURCE_NAME; // Discogs, MusicBrainz, ... ; needed by MainFormDlgImpl;
     //dest.m_imageInfo; // !!! not set
 }
 
 
+std::string DiscogsAlbumInfo::getGenre() const // combination of m_strGenre and m_strStyle
+{
+    switch (*m_peStyleOption)
+    {
+    case GENRE_ONLY: return m_strGenre;
+    case GENRE_COMMA_STYLE: return m_strGenre.empty() || m_strStyle.empty() ? m_strGenre + m_strStyle : m_strGenre + ", " + m_strStyle;
+    case GENRE_PAR_STYLE:
+        {
+            if (m_strStyle.empty()) { return m_strGenre; }
+            if (m_strGenre.empty()) { return "(" + m_strStyle + ")"; } // ttt2 perhaps without "(/)"
+            return m_strGenre + " (" + m_strStyle + ")";
+        }
+    case STYLE_ONLY: return m_strStyle;
+    }
+
+    CB_ASSERT (false);
+}
+
 }; // namespace Discogs
 
 
@@ -467,7 +493,7 @@ private:
 
 /*override*/ void DiscogsDownloader::saveSize()
 {
-    m_settings.saveDiscogsSettings(width(), height());
+    m_settings.saveDiscogsSettings(width(), height(), m_pStyleCbB->currentIndex());
 }
 
 
@@ -479,8 +505,14 @@ DiscogsDownloader::DiscogsDownloader(QWidget* pParent, SessionSettings& settings
 {
     setWindowTitle("Download album data from Discogs.com");
 
-    int nWidth, nHeight;
-    m_settings.loadDiscogsSettings(nWidth, nHeight);
+    int nWidth, nHeight, nStyleOption;
+    m_settings.loadDiscogsSettings(nWidth, nHeight, nStyleOption);
+    if (nStyleOption < 0 || nStyleOption > Discogs::DiscogsAlbumInfo::STYLE_ONLY)
+    {
+        nStyleOption = Discogs::DiscogsAlbumInfo::GENRE_ONLY;
+    }
+    m_eStyleOption = Discogs::DiscogsAlbumInfo::StyleOption(nStyleOption);
+
     if (nWidth > 400 && nHeight > 400) { resize(nWidth, nHeight); }
 
     delete m_pSrchArtistL;
@@ -491,12 +523,21 @@ DiscogsDownloader::DiscogsDownloader(QWidget* pParent, SessionSettings& settings
     delete m_pMatchCountCkB;
     m_pViewAtAmazonL->hide();
 
+    {
+        QStringList l;
+        //l << "< don't use >" << "Genre1, Genre2, ... , Style1, Style2, ..." << "Genre1, Genre2, ... (Style1, Style2, ...)" << "Style1, Style2, ...";
+        l << "Genres" << "Genres, Styles" << "Genres (Styles)" << "Styles";
+        m_pStyleCbB->addItems(l);
+        m_pStyleCbB->setCurrentIndex(nStyleOption);
+    }
+
     m_pQHttp->setHost("www.discogs.com");
 
     m_pModel = new WebDwnldModel(*this, *m_pTrackListG); // !!! in a way these would make sense to be in the base constructor, but that would cause calls to pure virtual methods
     m_pTrackListG->setModel(m_pModel);
 
     connect(m_pSearchB, SIGNAL(clicked()), this, SLOT(on_m_pSearchB_clicked()));
+    connect(m_pStyleCbB, SIGNAL(currentIndexChanged(int)), this, SLOT(on_m_pStyleCbB_currentIndexChanged(int)));
 }
 
 
@@ -508,6 +549,7 @@ DiscogsDownloader::~DiscogsDownloader()
 
 void DiscogsDownloader::clear()
 {
+LAST_STEP("DiscogsDownloader::clear");
     clearPtrContainer(m_vpImages);
     m_vAlbums.clear();
 }
@@ -515,6 +557,7 @@ void DiscogsDownloader::clear()
 
 /*override*/ bool DiscogsDownloader::initSearch(const std::string& strArtist, const std::string& strAlbum)
 {
+LAST_STEP("DiscogsDownloader::initSearch");
     string s (!strArtist.empty() && !strAlbum.empty() ? strArtist + " " + strAlbum : strArtist + strAlbum);
     s = removeParentheses(s);
 
@@ -528,8 +571,9 @@ void DiscogsDownloader::clear()
 // "/search?type=all&q=Beatles+Abbey+Road&f=xml&api_key=f51e9c8f6c"
 /*override*/ std::string DiscogsDownloader::createQuery()
 {
+LAST_STEP("DiscogsDownloader::createQuery");
     //string s (strArtist + "+" + strAlbum);
-    string s ("/search?type=all&q=" + convStr(m_pSrchArtistE->text()) + "&f=xml&api_key=f51e9c8f6c");
+    string s ("/search?type=all&q=" + replaceSymbols(convStr(m_pSrchArtistE->text())) + "&f=xml&api_key=f51e9c8f6c");
     for (string::size_type i = 0; i < s.size(); ++i)
     {
         if (' ' == s[i])
@@ -547,6 +591,7 @@ void DiscogsDownloader::clear()
 
 void DiscogsDownloader::on_m_pSearchB_clicked()
 {
+LAST_STEP("DiscogsDownloader::on_m_pSearchB_clicked");
     clear();
     search();
 }
@@ -560,6 +605,7 @@ void DiscogsDownloader::on_m_pSearchB_clicked()
 
 void DiscogsDownloader::loadNextPage()
 {
+LAST_STEP("DiscogsDownloader::loadNextPage");
     CB_ASSERT (!m_pQHttp->hasPendingRequests());
     ++m_nLastLoadedPage;
     CB_ASSERT (m_nLastLoadedPage <= m_nTotalPages - 1);
@@ -581,18 +627,21 @@ void DiscogsDownloader::loadNextPage()
 
 void DiscogsDownloader::reloadGui()
 {
+LAST_STEP("DiscogsDownloader::reloadGui");
     AlbumInfoDownloaderDlgImpl::reloadGui();
 
     const DiscogsAlbumInfo& album (m_vAlbums[m_nCrtAlbum]);
     m_pAlbumNotesM->setText(convStr(album.m_strNotes));
-    m_pGenreE->setText(convStr(album.m_strGenre));
+    m_pGenreE->setText(convStr(album.getGenre()));
 }
 
 
 
 void DiscogsDownloader::requestAlbum(int nAlbum)
 {
-    CB_ASSERT (!m_pQHttp->hasPendingRequests());
+LAST_STEP("DiscogsDownloader::requestAlbum");
+    CB_ASSERT (!m_pQHttp->hasPendingRequests()); //ttt1 triggered according to mail (Nov 2, 2009, 2:31 PM - Qt 4.5.2; there is something in the 4.5.3 change log at http://qt.nokia.com/developer/changes/changes-4.5.3 about duplicate HTTP requests, but according to http://www.qtcentre.org/forum/f-qt-programming-2/t-qhttp-response-content-25255-post121265.html it only affected QNetworkAccessManager, which is a replacement for QHttp, so this is probably not related)
+
     m_nLoadingAlbum = nAlbum;
     setWaiting(ALBUM);
     string s ("/release/" + m_vAlbums[nAlbum].m_strId + "?f=xml&api_key=f51e9c8f6c");
@@ -608,6 +657,7 @@ void DiscogsDownloader::requestAlbum(int nAlbum)
 
 void DiscogsDownloader::requestImage(int nAlbum, int nImage)
 {
+LAST_STEP("DiscogsDownloader::requestImage");
     CB_ASSERT (!m_pQHttp->hasPendingRequests());
     m_nLoadingAlbum = nAlbum;
     m_nLoadingImage = nImage;
@@ -627,6 +677,16 @@ void DiscogsDownloader::requestImage(int nAlbum, int nImage)
 }
 
 
+void DiscogsDownloader::on_m_pStyleCbB_currentIndexChanged(int k)
+{
+LAST_STEP("DiscogsDownloader::on_m_pStyleCbB_currentIndexChanged");
+    if (m_nCrtAlbum < 0 || m_nCrtAlbum >= cSize(m_vAlbums)) { return; }
+    DiscogsAlbumInfo& album (m_vAlbums[m_nCrtAlbum]);
+    m_eStyleOption = Discogs::DiscogsAlbumInfo::StyleOption(k);
+    m_pGenreE->setText(convStr(album.getGenre()));
+}
+
+
 //==========================================================================================================================
 //==========================================================================================================================
 //==========================================================================================================================
@@ -635,6 +695,7 @@ void DiscogsDownloader::requestImage(int nAlbum, int nImage)
 
 /*override*/ QHttp* DiscogsDownloader::getWaitingHttp()
 {
+LAST_STEP("DiscogsDownloader::getWaitingHttp");
     return m_pQHttp;
 }
 
@@ -667,4 +728,4 @@ void DiscogsDownloader::requestImage(int nAlbum, int nImage)
 
 
 
-//ttt1 perhaps make m_pResArtistE editable
+
