@@ -284,7 +284,7 @@ long getMemUsage()
     int n ((int)getpid());
     char a [30];
     sprintf (a, "/proc/%d/status", n); // ttt2 linux-specific; not sure how version-specific this is;
-    // sprintf (a, "/proc/self/status", n); // ttt1 try this (after checking portability)
+    // sprintf (a, "/proc/self/status", n); // ttt2 try this (after checking portability)
     ifstream_utf8 in (a);
     string s;
     while (getline(in, s))
@@ -374,7 +374,9 @@ namespace
         const char* getSzLayer() const;
         const char* getSzChannelMode() const;
 
+        string initialize(const unsigned char* bfr, bool* pbIsValid);
         string decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid);
+        string decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid);
     };
 
     const char* Decoder::getSzVersion() const
@@ -395,7 +397,8 @@ namespace
         return s_channelModeName[m_eChannelMode];
     }
 
-    string Decoder::decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid) //ttt1 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
+
+    string Decoder::initialize(const unsigned char* bfr, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
     {
         bool b;
         bool& bRes (0 == pbIsValid ? b : *pbIsValid);
@@ -409,7 +412,7 @@ namespace
             int nVer ((*pHeader & 0x18) >> 3);
             switch (nVer)
             {//TRACE
-            case 0x00: bRes = false; return "Not an MPEG frame. Unsupported version (2.5)."; //ttt1 see about supporting this: search for MPEG1 to find other places
+            case 0x00: bRes = false; return "Not an MPEG frame. Unsupported version (2.5)."; //ttt2 see about supporting this: search for MPEG1 to find other places
                 // ttt2 in a way it would make more sense to warn that it's not supported, with "MP3_THROW(SUPPORT, ...)", but before warn, make sure it's a valid 2.5 frame, followed by another frame ...
 
             case 0x02: m_eVersion = MPEG2; break;
@@ -518,6 +521,15 @@ namespace
         default: throw 1; // it should have thrown before getting here
         }
 
+        return "";
+    }
+
+
+    string Decoder::decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
+    {
+        string s (initialize(bfr, pbIsValid));
+        if (!s.empty()) { return s; }
+
         ostringstream out;
         /*out << getSzVersion() << " " << getSzLayer() << ", " << m_nBitrate/1000 << "kbps, " << m_nFrequency << "Hz, " << getSzChannelMode() << ", padding=" << (m_nPadding ? "true" : "false") << ", length " <<
                 m_nSize << " (0x" << hex << m_nSize << dec << ")";*/
@@ -528,6 +540,25 @@ namespace
         return out.str();
     }
 
+
+    string Decoder::decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
+    {
+        string s (initialize(bfr, pbIsValid));
+        if (!s.empty()) { return s; }
+
+        ostringstream out;
+        out << " version=\"" << getSzVersion() << "\""
+            << " layer=\"" << getSzLayer() << "\""
+            << " channelMode=\"" << getSzChannelMode() << "\""
+            << " frequency=\"" << m_nFrequency << "\""
+            << " bps=\"" << m_nBitrate << "\""
+            << " crc=\"" << boolAsYesNo(m_bCrc) << "\""
+
+            << " mpegSize=\"" << m_nSize << "\""
+            << " padding=\"" << boolAsYesNo(m_nPadding) << "\"";
+
+        return out.str();
+    }
 } // namespace
 
 
@@ -551,6 +582,12 @@ string decodeMpegFrame(const char* bfr, const char* szSep, bool* pbIsValid /*= 0
 }
 
 
+string decodeMpegFrameAsXml(const char* bfr, bool* pbIsValid /*= 0*/)
+{
+    Decoder d;
+    const unsigned char* q (reinterpret_cast<const unsigned char*>(bfr));
+    return d.decodeMpegFrameAsXml(q, pbIsValid);
+}
 
 
 StreamStateRestorer::StreamStateRestorer(istream& in) : m_in(in), m_pos(in.tellg()), m_bOk(false)
@@ -630,6 +667,8 @@ string escapeHttp(const string& s)
 }
 
 
+
+
 vector<string> convStr(const vector<QString>& v)
 {
     vector<string> u;
@@ -666,7 +705,7 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 */
 
 #ifndef WIN32
-    Qt::WindowFlags getMainWndFlags() { return Qt::WindowTitleHint; } // !!! these are incorrect, but seem the best option; the values used for Windows are supposed to be OK; they work as expected with KDE but not with Gnome (asking for maximize button not only fails to sho it, but causes the "Close" button to disappear as well); Since in KDE min/max buttons are shown when needed anyway, it's sort of OK // ttt0 see if there is workaround/fix
+    Qt::WindowFlags getMainWndFlags() { return Qt::WindowTitleHint; } // !!! these are incorrect, but seem the best option; the values used for Windows are supposed to be OK; they work as expected with KDE but not with Gnome (asking for maximize button not only fails to sho it, but causes the "Close" button to disappear as well); Since in KDE min/max buttons are shown when needed anyway, it's sort of OK // ttt2 see if there is workaround/fix
     Qt::WindowFlags getDialogWndFlags() { return Qt::WindowTitleHint; }
     Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; }
 #else
@@ -677,7 +716,7 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 
 
 #if 0
-//ttt1 add desktop, distribution, WM, ...
+//ttt2 add desktop, distribution, WM, ...
 #ifndef WIN32
     utsname info;
     uname(&info);
@@ -710,7 +749,7 @@ static void removeStr(string& main, const string& sub)
 extern const char* APP_VER;
 
 
-QString getSystemInfo() //ttt1 perhaps store this at startup, so fewer things may go wrong fhen the assertion handler needs it
+QString getSystemInfo() //ttt2 perhaps store this at startup, so fewer things may go wrong fhen the assertion handler needs it
 {
     QString s ("OS: ");
 
@@ -728,7 +767,7 @@ QString getSystemInfo() //ttt1 perhaps store this at startup, so fewer things ma
     s += utsInfo.version; s += " ";
     for (int i = 0; i < lFiles.size(); ++i)
     {
-        //qDebug("%s", lFiles[i].toUtf8().data());
+        //qDebug("%s", lFiles[i].toUtf8().constData());
         if ("lsb-release" != lFiles[i])
         {
             QFile f ("/etc/" + lFiles[i]);
@@ -745,7 +784,7 @@ QString getSystemInfo() //ttt1 perhaps store this at startup, so fewer things ma
     if (f.open(QIODevice::ReadOnly))
     {
         QByteArray b (f.read(1000));
-        string s1 (b.data());
+        string s1 (b.constData());
 
         removeStr(s1, "Welcome to");
         removeStr(s1, "Kernel");
@@ -765,16 +804,16 @@ QString getSystemInfo() //ttt1 perhaps store this at startup, so fewer things ma
         }
 
         s += convStr(s1);
-//qDebug("a: %s", s.toUtf8().data());
+//qDebug("a: %s", s.toUtf8().constData());
         /*for (;;)
         {
             string::size_type n (s.find('\n'));
             if (string::npos == n) { break; }
             s1[n] = ' ';
         }*/
-//qDebug("b: %s", s.toUtf8().data());
+//qDebug("b: %s", s.toUtf8().constData());
     }
-//ttt0 search /proc for kwin, metacity, ...
+//ttt2 search /proc for kwin, metacity, ...
 
 
 #else
@@ -879,7 +918,7 @@ void configureGradient(QGradient& grad, const QColor& col, double dStart, double
         }
     }
 #else
-    grad.setColorAt(0, col.lighter(dStart < 0.0001 ? 119 : 100)); //ttt1 perhaps use this or at least add an option
+    grad.setColorAt(0, col.lighter(dStart < 0.0001 ? 119 : 100)); //ttt2 perhaps use this or at least add an option
     grad.setColorAt(0.48, col);
     grad.setColorAt(0.52, col);
     grad.setColorAt(1, col.lighter(dEnd > 0.9999 ? 80 : 100));
@@ -895,16 +934,16 @@ vector<QString> getLocalHelpDirs()
         s_v.push_back("/home/ciobi/cpp/Mp3Utils/mp3diags/doc/html/");
         s_v.push_back("/usr/share/mp3diags-doc/html/");
         s_v.push_back("/usr/share/doc/mp3diags/html/");
-        s_v.push_back("/usr/share/doc/mp3diags-0.99.05.033/html/");
+        s_v.push_back("/usr/share/doc/mp3diags-1.0.05.050/html/");
 #else
         wchar_t wszModule [200];
         int nRes (GetModuleFileName(0, wszModule, 200));
-        //qDebug("%s", QString::fromWCharArray(wszModule).toUtf8().data());
+        //qDebug("%s", QString::fromWCharArray(wszModule).toUtf8().constData());
         if (0 < nRes && nRes < 200)
         {
             s_v.push_back(QFileInfo(
                     fromNativeSeparators(QString::fromWCharArray(wszModule))).dir().absolutePath() + "/doc/");
-            //qDebug("%s", s_v.back().toUtf8().data());
+            //qDebug("%s", s_v.back().toUtf8().constData());
         }
 #endif
     }
@@ -931,7 +970,7 @@ void openHelp(const string& strFileName)
     QString qs (strDir);
     if (qs.isEmpty())
     {
-        qs = "http://mp3diags.sourceforge.net/";
+        qs = "http://mp3diags.sourceforge.net/"; //ttt1 different for "unstable"
     }
     else
     {
@@ -941,8 +980,8 @@ void openHelp(const string& strFileName)
     qs = qs + convStr(strFileName);
 
 
-//qDebug("open %s", qs.toUtf8().data());
-//logToGlobalFile(qs.toUtf8().data());
+//qDebug("open %s", qs.toUtf8().constData());
+//logToGlobalFile(qs.toUtf8().constData());
     CursorOverrider ovr;
     QDesktopServices::openUrl(QUrl(qs, QUrl::TolerantMode));
 }
