@@ -174,7 +174,7 @@ MpegStream::MpegStream(int nIndex, NoteColl& notes, istream& in) : MpegStreamBas
         MP3_THROW (m_pos, audioTooShort, StreamTooShort(strInfo, m_nFrameCount));
     }
 
-    MP3_CHECK (!m_bVbr || bVbr2, m_pos, diffBitrateInFirstFrame, UnknownHeader()); //ttt1 perhaps add test for "null": whatever is in the first bytes that allows Xing & Co to not generate audio in decoders that don't know about them
+    MP3_CHECK (!m_bVbr || bVbr2, m_pos, diffBitrateInFirstFrame, UnknownHeader()); //ttt2 perhaps add test for "null": whatever is in the first bytes that allows Xing & Co to not generate audio in decoders that don't know about them
 
     m_nSize = pos - m_pos;
     in.seekg(pos);
@@ -221,18 +221,22 @@ void MpegStream::removeLastFrame()
 /*override*/ std::string MpegStream::getInfo() const
 {
     ostringstream out;
-    int nDur (int(m_nSize*8.0/m_nBitrate));
-    int nMin (nDur/60);
-    int nSec (nDur - nMin*60);
-    char a [15];
-    sprintf(a, "%d:%02d", nMin, nSec);
-    out << a << ", " << m_firstFrame.getSzVersion() << " " << m_firstFrame.getSzLayer() << ", " << m_firstFrame.getSzChannelMode() << ", " <<
+    out << getDuration() << ", " << m_firstFrame.getSzVersion() << " " << m_firstFrame.getSzLayer() << ", " << m_firstFrame.getSzChannelMode() << ", " <<
         m_firstFrame.getFrequency() << "Hz, " << m_nBitrate << "bps " << (m_bVbr ? "VBR" : "CBR") << ", CRC=" <<
         boolAsYesNo(m_firstFrame.getCrcUsage()) << ", frame count=" << m_nFrameCount;
     out << "; last frame" << (m_bRemoveLastFrameCalled ? " removed; it was" : "") << " located at 0x" << hex << m_posLastFrame << dec;
     return out.str();
 }
 
+std::string MpegStream::getDuration() const
+{
+    int nDur (int(m_nSize*8.0/m_nBitrate));
+    int nMin (nDur/60);
+    int nSec (nDur - nMin*60);
+    char a [15];
+    sprintf(a, "%d:%02d", nMin, nSec);
+    return a;
+}
 
 bool MpegStream::isCompatible(const MpegFrameBase& frame)
 {
@@ -314,14 +318,17 @@ XingStreamBase::XingStreamBase(int nIndex, NoteColl& notes, istream& in) : MpegS
     in.seekg(m_pos);
 
     const int XING_LABEL_SIZE (4);
-    const int BFR_SIZE (MpegFrame::MPEG_FRAME_HDR_SIZE + 32 + XING_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt1 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
+    const int BFR_SIZE (MpegFrame::MPEG_FRAME_HDR_SIZE + 32 + XING_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt2 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
     char bfr [BFR_SIZE];
 
     int nSideInfoSize (m_firstFrame.getSideInfoSize());
 
     int nBfrSize (MpegFrame::MPEG_FRAME_HDR_SIZE + nSideInfoSize + XING_LABEL_SIZE);
+
+    MP3_CHECK_T (nBfrSize <= m_firstFrame.getSize(), m_pos, "Not a Xing stream. This kind of MPEG audio doesn't support Xing.", NotXingStream()); // !!! some kinds of MPEG audio (e.g. "MPEG-1 Layer I, 44100Hz 32000bps" or "MPEG-2 Layer III, 22050Hz 8000bps") have very short frames, which can't accomodate a Xing header
+
     streamsize nRead (read(in, bfr, nBfrSize));
-    STRM_ASSERT (nBfrSize == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown)
+    STRM_ASSERT (nBfrSize == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown) and nBfrSize is no bigger than the frame
 
     char* pLabel (bfr + MpegFrame::MPEG_FRAME_HDR_SIZE + nSideInfoSize);
     MP3_CHECK_T (0 == strncmp("Xing", pLabel, XING_LABEL_SIZE) || 0 == strncmp("Info", pLabel, XING_LABEL_SIZE), m_pos, "Not a Xing stream. Header not found.", NotXingStream());
@@ -363,12 +370,22 @@ void XingStreamBase::getXingInfo(std::ostream& out) const
     out << "[Xing header info:";
     bool b (false);
     if (0x01 == (m_cFlags & 0x01)) { out << " frame count=" << m_nFrameCount; b = true; }
-    if (0x02 == (m_cFlags & 0x02)) { out << (b ? "," : "") << " byte count=" << m_nByteCount; b = true; } //ttt1 see what to do with this: it's the size of the whole file, all headers&tags included (at least with c03 Valentin Moldovan - Marea Irlandei.mp3); ??? and anyway,  what's the point of including the size of the whole file as a field?
+    if (0x02 == (m_cFlags & 0x02)) { out << (b ? "," : "") << " byte count=" << m_nByteCount; b = true; } //ttt2 see what to do with this: it's the size of the whole file, all headers&tags included (at least with c03 Valentin Moldovan - Marea Irlandei.mp3); ??? and anyway,  what's the point of including the size of the whole file as a field?
     if (0x04 == (m_cFlags & 0x04)) { out << (b ? "," : "") << " TOC present"; b = true; }
     if (0x08 == (m_cFlags & 0x08)) { out << (b ? "," : "") << " quality=" << m_nQuality; b = true; }
     out << "]";
 }
 
+std::string XingStreamBase::getInfoForXml() const
+{
+    ostringstream out;
+    if (0x01 == (m_cFlags & 0x01)) { out << " frameCount=\"" << m_nFrameCount << "\""; }
+    if (0x02 == (m_cFlags & 0x02)) { out << " byteCount=\"" << m_nByteCount << "\""; } //ttt2 see what to do with this: it's the size of the whole file, all headers&tags included (at least with c03 Valentin Moldovan - Marea Irlandei.mp3); ??? and anyway,  what's the point of including the size of the whole file as a field?
+    if (0x04 == (m_cFlags & 0x04)) { out << " toc=\"yes\""; }
+    if (0x08 == (m_cFlags & 0x08)) { out << " quality=\"" << m_nQuality << "\""; }
+    return out.str();
+}
+
 
 /*override*/ std::string XingStreamBase::getInfo() const
 {
@@ -431,11 +448,13 @@ LameStream::LameStream(int nIndex, NoteColl& notes, istream& in) : XingStreamBas
 
     const int LAME_LABEL_SIZE (4);
     const int LAME_OFFS (156);
-    const int BFR_SIZE (LAME_OFFS + LAME_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt1 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
+    const int BFR_SIZE (LAME_OFFS + LAME_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt2 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
     char bfr [BFR_SIZE];
 
+    MP3_CHECK_T (BFR_SIZE <= m_firstFrame.getSize(), m_pos, "Not a LAME stream. This kind of MPEG audio doesn't support LAME.", NotLameStream()); // !!! some kinds of MPEG audio have very short frames, which can't accomodate a VBRI header
+
     streamsize nRead (read(in, bfr, BFR_SIZE));
-    STRM_ASSERT (BFR_SIZE == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown)
+    STRM_ASSERT (BFR_SIZE == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown) and BFR_SIZE is no bigger than the frame
 
     MP3_CHECK_T (0 == strncmp("LAME", bfr + LAME_OFFS, LAME_LABEL_SIZE), m_pos, "Not a LAME stream. Header not found.", NotLameStream());
 
@@ -460,16 +479,18 @@ LameStream::LameStream(int nIndex, NoteColl& notes, istream& in) : XingStreamBas
 
 
 
-//ttt1 see why after most VBRI headers comes an "unknown" stream; perhaps there's an error in how VbriStream works
+//ttt2 see why after most VBRI headers comes an "unknown" stream; perhaps there's an error in how VbriStream works
 VbriStream::VbriStream(int nIndex, NoteColl& notes, istream& in) : MpegStreamBase(nIndex, notes, in)
 {
     in.seekg(m_pos);
     const int VBRI_LABEL_SIZE (4);
-    const int BFR_SIZE (MpegFrame::MPEG_FRAME_HDR_SIZE + 32 + VBRI_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt1 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
+    const int BFR_SIZE (MpegFrame::MPEG_FRAME_HDR_SIZE + 32 + VBRI_LABEL_SIZE); // MPEG header + side info + "Xing" size //ttt2 not sure if space for CRC16 should be added; then not sure if frame size should be increased by 2 when CRC is found
     char bfr [BFR_SIZE];
 
+    MP3_CHECK_T (BFR_SIZE <= m_firstFrame.getSize(), m_pos, "Not a VBRI stream. This kind of MPEG audio doesn't support VBRI.", NotVbriStream()); // !!! some kinds of MPEG audio have very short frames, which can't accomodate a VBRI header
+
     streamsize nRead (read(in, bfr, BFR_SIZE));
-    STRM_ASSERT (BFR_SIZE == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown) // ttt0 was reported as triggered at https://sourceforge.net/apps/mantisbt/mp3diags/view.php?id=8
+    STRM_ASSERT (BFR_SIZE == nRead); // this was supposed to be a valid frame to begin with (otherwise the base class would have thrown) and BFR_SIZE is no bigger than the frame
 
     char* pLabel (bfr + MpegFrame::MPEG_FRAME_HDR_SIZE + 32);
     MP3_CHECK_T (0 == strncmp("VBRI", pLabel, VBRI_LABEL_SIZE), m_pos, "Not a VBRI stream. Header not found.", NotVbriStream());
@@ -508,7 +529,7 @@ Id3V1Stream::Id3V1Stream(int nIndex, NoteColl& notes, istream& in) : DataStream(
 
     const int BFR_SIZE (128);
     streamsize nRead (read(in, m_data, BFR_SIZE));
-    MP3_CHECK_T (nRead >= 3, m_pos, "Invalid ID3V1 tag. File too short.", NotId3V1Stream());
+    MP3_CHECK_T (BFR_SIZE == nRead, m_pos, "Invalid ID3V1 tag. File too short.", NotId3V1Stream());
     MP3_CHECK_T (0 == strncmp("TAG", m_data, 3), m_pos, "Invalid ID3V1 tag. Invalid header.", NotId3V1Stream());
 
     MP3_CHECK (BFR_SIZE == nRead, m_pos, id3v1TooShort, NotId3V1Stream());
@@ -529,7 +550,7 @@ Id3V1Stream::Id3V1Stream(int nIndex, NoteColl& notes, istream& in) : DataStream(
     TestResult eArtist (checkId3V1String(m_data + 33, 30)); MP3_CHECK (BAD != eArtist, m_pos, id3v1InvalidArtist, NotId3V1Stream());
     TestResult eAlbum (checkId3V1String(m_data + 63, 30)); MP3_CHECK (BAD != eAlbum, m_pos, id3v1InvalidAlbum, NotId3V1Stream());
     TestResult eYear (checkId3V1String(m_data + 93, 4)); MP3_CHECK (BAD != eYear, m_pos, id3v1InvalidYear, NotId3V1Stream());
-    TestResult eComment (checkId3V1String(m_data + 97, 28)); MP3_CHECK (BAD != eComment, m_pos, id3v1InvalidComment, NotId3V1Stream()); // "28" is for ID3V1.1b (there's no reliable way to distinguish among versions 1.0 and 1.1 by design, and in practice among any of them because some tools use 0 instead of space and 0 seems to be a valid value for 1.1b's track and genre, for "undefined") //ttt1 use m_eVersion
+    TestResult eComment (checkId3V1String(m_data + 97, 28)); MP3_CHECK (BAD != eComment, m_pos, id3v1InvalidComment, NotId3V1Stream()); // "28" is for ID3V1.1b (there's no reliable way to distinguish among versions 1.0 and 1.1 by design, and in practice among any of them because some tools use 0 instead of space and 0 seems to be a valid value for 1.1b's track and genre, for "undefined") //ttt2 use m_eVersion
 
     if (ZERO_PADDED == eTrack || ZERO_PADDED == eArtist || ZERO_PADDED == eAlbum || ZERO_PADDED == eYear || ZERO_PADDED == eComment)
     {
@@ -603,17 +624,22 @@ static string getSpacedStr(const string& s)
     return ", " + s;
 }
 
-/*override*/ std::string Id3V1Stream::getInfo() const
+
+const char* Id3V1Stream::getVersion() const
 {
-    string strRes;
     switch (m_eVersion)
     {
-    case V10: strRes = "ID3V1.0"; break;
-    case V11: strRes = "ID3V1.1"; break;
-    case V11b: strRes = "ID3V1.1b"; break;
+    case V10: return "ID3V1.0";
+    case V11: return "ID3V1.1";
+    case V11b: return "ID3V1.1b";
     default:
         STRM_ASSERT (false);
     }
+}
+
+/*override*/ std::string Id3V1Stream::getInfo() const
+{
+    string strRes (getVersion());
     strRes += getSpacedStr(getTitle(0)) + getSpacedStr(getArtist(0)) + getSpacedStr(getAlbumName(0)) + getSpacedStr(getGenre(0));
     return strRes;
 }
@@ -819,7 +845,7 @@ const char* getId3V1Genre(int n)
         "Native US",
         "Cabaret",
         "New Wave",
-        "Psychadelic",
+        "Psychedelic", // ??? "Psychadelic" in the specs
         "Rave",
         "Showtunes",
         "Trailer",
@@ -915,7 +941,4 @@ const char* getId3V1Genre(int n)
 
 
 
-//ttt1 see how adding a Xing frame affects gapless playing (seems that this can be checked on "Lasa, lasa Nr.5" -> "Dansul codrilor"
-
-//ttt1 warning on low bitrate or other "low quality" settings
 
